import logging
import os
import json
from p2.driver.k8s import k8s
from p2.driver.k8s.k8s import get_current_context
from p2.store.client import Client


class AccessReport:


    def __init__(self):
        p2_store_url = os.environ.get('P2_STORE_URL')

        self.log = logging.getLogger(__name__)
        self.client = Client(p2_store_url, timeout=10)
        self.config = self.client.get_config('p2.reporter.access')


    def store_reports(self):
        """
        Workflow:
        * get reports
        * update p2.store.report identified by config.report type/title
        """
        try:
            reports = self.get_reports()
            for name in reports:
                record={
                    **reports[name]['config']['report'],
                    'params': json.dumps(reports[name]['config']),
                    'body': json.dumps(reports[name]['records'], sort_keys=True)
                    }
                self.client.add('reports', record)
                self.log.info(f"""Report {reports[name]['config']['report']['title']}
                        generated and stored""")
            return 1
        except Exception as e:
            self.log.error("Something went wrong here...: {}".format(e))
            return None

    def get_reports(self):
        """
        Workflow:
        * get current context
        * initialize access_review_status
        * get all namespaces matching namespaces section
        * for each namespace:
            * for each rule:
                * for each rule.resources:
                    * for each rule.verbs:
                        * get access review
                        * add access review status to access_review_status
        """
        context = get_current_context()
        access_review_status = dict()
        self.log.info(f"Getting reports for context {context['name']}")
        k8_api = k8s.Api()
        context_config = self.config.get(context['name'])
        if context_config is None:
            self.log.error(f"No configuration found for context {context['name']}")
        name=context['name']
        access_review_status[name] = dict()
        access_review_status[name]["config"] = context_config
        access_review_status[name]["records"] = list()
        self.log.info("Getting all namespaces matching pattern {}...".format(
            context_config["namespaces"]["pattern"]))
        namespaces = k8_api.namespaces(pattern=context_config["namespaces"]["pattern"])
        self.log.info("Found {} total namespaces...".format(len(namespaces)))
        for namespace in namespaces:
            for to_skip in context_config["namespaces"]["skip"]:
                if to_skip in namespace["name"]:
                    namespaces.remove(namespace)
        self.log.info("After filtering out: {}...".format(len(namespaces)))
        for namespace in namespaces:
            self.log.info(f"Getting access status on namespace: {namespace}...")
            for rule in context_config["rules"]:
                for resource in rule["resources"]:
                    for verb in rule["verbs"]:
                        record={
                            "area": namespace["area"],
                            "area_owner": namespace.get('area_owner', 'not set'),
                            "team": namespace.get('team', 'not set'),
                            "area_owner_email": namespace["area_owner_email"],
                            "namespace": namespace["name"],
                            "context": name,
                            "resource": resource,
                            "verb": verb,
                            "allowed": k8_api.access_review(
                                namespace=namespace["name"],
                                resource=resource,
                                verb=verb
                            )
                        }
                        self.log.info("Report for namespace {}, resource {}, verb {}: {}".format(
                            namespace, resource, verb, record["allowed"]))
                        access_review_status[name]["records"].append(record)
        return access_review_status
